#!/usr/bin/perl -w

# use it's own email address for answering, with IMAP

use Manager::Dialog qw(Approve ApproveCommands);
use PerlLib::Cacher;
use PerlLib::MySQL;
use PerlLib::SwissArmyKnife;
use PerlLib::ToText;

use DateTime;
use File::Temp qw(tempdir);
use FileHandle;
use Text::Wrap;
use Tk;
use Tk::BrowseEntry;
use XML::Simple;

my $rssfile = "/var/lib/myfrdcsa/codebases/minor/js-rapid-response/data/index.rss";
my $c = read_file($rssfile);
my $xml = XMLin($c);

# go ahead and update the database with the new entries

# repeated load this

# play music or rather, add a notification to the notification manager
# if there are new items

my $cacher = PerlLib::Cacher->new;
my $totext = PerlLib::ToText->new;
my $mysql = PerlLib::MySQL->new
  (
   DBName => "js_rapid_response",
  ) if 0;
my $top1 = MainWindow->new
  (
   -title => "Job-Search Rapid Response Tool",
   -height => 600,
   -width => 800,
  );
$UNIVERSAL::managerdialogtkwindow = $top1;

$frame = $top1->Frame(-relief => 'flat');
$frame->pack(-side => 'top', -fill => 'both', -expand => 1, -anchor => 'center');

$menu = $top1->Frame(-relief => 'raised', -borderwidth => '1');
$menu->pack(-before => $frame, -side => 'top', -fill => 'x');

$menu_file = $menu->Menubutton(-text => 'File', -underline => 0);
$menu_file->command(-label => 'load ...', -command => sub { },
		    -underline => 0);
$menu_file->command(-label => 'Exit', -command => sub { }, -underline => 0);
$menu_file->pack(-side => 'left');

my $scrollframe = $top1->Frame;
my $box = $scrollframe->Listbox
  (
   -relief => 'sunken',
   -width  => 40,
   -setgrid => 1,
  );
foreach my $item (@{$xml->{item}}) {
  $box->insert('end', $item->{title});
}
$box->bind('all', '<Control-c>' => \&exit);
$box->bind('<Button-1>' => sub {
	     my($listbox) = @_;
	     my $item = $listbox->get('active');
	     foreach my $entry (@{$xml->{item}}) {
	       if ($entry->{title} eq $item) {
		 # launch the window with the entry
		 ManageEntry
		   (
		    Entry => $entry,
		   );
	       }
	     }
	   });
my $scroll = $scrollframe->Scrollbar(-command => ['yview', $box]);
$box->configure(-yscrollcommand => ['set', $scroll]);
$box->pack(-side => 'left', -fill => 'both', -expand => 1);
$scroll->pack(-side => 'right', -fill => 'y');
$scrollframe->pack(-fill => 'both', -expand => 1);

$buttonframe = $top1->Frame();
$buttonframe->Button
  (
   -text => "Update",
   -command => sub { 
     Update();
   },
  )->pack(-side => "left");
$buttonframe->pack;
$buttonframe->Button
  (
   -text => "Exit",
   -command => sub { 
     if (Approve("Exit Program?")) {
       exit(0);
     }
   },
  )->pack(-side => "right");
$buttonframe->pack;

MainLoop;

sub ManageEntry {
  my %args = @_;
  my $top2 = $top1->Toplevel
    (
     -title => $args{Entry}->{title},
     -height => 600,
     -width => 800,
    );

  my $defaultlettertext = <<EOF1;
To whom it may concern,

Please consider my application for your position of <POSITION>.

Attached are my cover letter and resumes.

Thank you for your consideration,
Andrew Dougherty

GVoice:   (331) 684-7712
Mobile:   (630) 340-7712
E-mail:   andrewdo\@frdcsa.org

P.S. <POSTSCRIPTUM>
EOF1

  my $defaultcoverlettertext = <<EOF2;
To whom it may concern,

Please consider my application for your position of <POSITION>.

Sincerely,
Andrew Dougherty
EOF2

  # have a set of text boxes to display the common information

  # go ahead and display everything, add buttons for submitting the
  # job

  # populate the default letter

  # have a few different templates for different types of jobs

  # rate job similarity using existing job search stuff

  # auto attach resumes

  # also record the job application to the database
  my $order1 = [
		"title",
		"link",
		"description",
		"dc:date",
		"dc:language",
		"dc:rights",
		"dc:source",
		# "dc:title",
		"dc:type",
		"dcterms:issued",
	       ];
  my $fields1 = {
		 "title" => {
			     Type => "small",
			    },
		 "link" => {
			    Type => "small",
			   },
		 "description" => {
				   Type => "large",
				  },
		 "dc:date" => {
			       Type => "small",
			      },
		 "dc:language" => {
				   Type => "small",
				  },
		 "dc:rights" => {
				 Type => "small",
				},
		 "dc:source" => {
				 Type => "small",
				},
		 "dc:title" => {
				Type => "small",
			       },
		 "dc:type" => {
			       Type => "small",
			      },
		 "dcterms:issued" => {
				      Type => "small",
				     },
		};

  my $fieldframes = $top2->Frame()->pack(-side => "left");
  foreach my $field (@$order1) {
    my $var;
    if ($field eq "description") {
      my $res = $totext->ToText
	(
	 String => $args{Entry}->{$field},
	);
      if ($res->{Success}) {
	$var = $res->{Text};
      } else {
	$var = $args{Entry}->{$field};
      }
    } else {
      $var = $args{Entry}->{$field};
    }
    $fields1->{$field}->{EntryText} = \$var;
    my $fieldframe = $fieldframes->Frame();
    $fieldframe->Label
      (
       -text => $field,
      )->pack(-side => 'left');
    if ($fields1->{$field}->{Type} eq "large") {
      my $text = $fieldframe->Text
	(
	 -width => 80,
	 -height => 30,
	);
      # render this to text
      $text->Contents(${$fields1->{$field}->{EntryText}});
      $text->pack(-side => 'right');
    } else {
      $fieldframe->Entry
	(
	 -width => length(${$fields1->{$field}->{EntryText}}),
	 -textvariable => $fields1->{$field}->{EntryText},
	)->pack(-side => 'right');
    }
    $fieldframe->pack
      (-side => "top");
  }

  ########################################

  my $textframes = $top2->Frame();

  # ahead and add a drop down selector for different configurations
  # $textframes->DropDownMenu;


  my $dropdown_value;
  my $dropdown = $textframes->BrowseEntry
    (
     -variable => \$dropdown_value,
     -width => 80,
    )->pack;

  # Configure dropdown
  $dropdown->configure
    (
     # What to do when an entry is selected
     -browsecmd => sub {
       print Dumper([$dropdown_value]);
       $dropdown_value = "";
     },
    );
  $dropdown->bind
    (
     '<Return>' => sub {
       print Dumper([$dropdown_value]);
       $dropdown_value = "";
     },
    );

  my @items = ("Resume", "Resume with Cover Letter");

  # Populate dropdown with values
  foreach my $sampletask (@items) {
    $dropdown->insert('end',$sampletask);
  }

  my $order2 = [
		"To:",
		"Cc:",
		"Bcc:",
		"Subject:",
		"From:",
		# "Fcc: +backup",
		"Body:",
		"Cover",
	       ];
  my $fields2 = {
		 "To:" => {
			   Type => "small",
			   Default => $args{Entry}->{"dc:source"},
			  },
		 "Cc:" => {
			   Type => "small",
			   Default => "",
			  },
		 "Bcc:" => {
			    Type => "small",
			    Default => "",
			   },
		 "Subject:" => {
				Type => "small",
				Default => "Re: ".$args{Entry}->{title},
			       },
		 "From:" => {
			     Type => "small",
			     Default => "Andrew Dougherty <andrewdo\@frdcsa.org>",
			    },
		 "Body:" => {
			     Type => "large",
			     Default => $defaultlettertext,
			    },
		 "Cover" => {
			     Type => "large",
			     Default => $defaultcoverlettertext,
			    },
		};
  foreach my $field (@$order2) {
    my $var;
    if ($field eq "Body:" or $field eq "Cover") {
      my $t = $fields2->{$field}->{Default};
      my $position = $args{Entry}->{title};
      my $url = $args{Entry}->{"dc:source"};
      $t =~ s/<POSITION>/$position/g;
      $t =~ s/<POSTSCRIPTUM>/This is in response to:\n$url/g;
      $var = $t;
    } elsif ($field eq "To:") {
      # have to retrieve the webpage and extract the email
      my $url = $args{Entry}->{"dc:source"};
      $cacher->get($url);
      my $content = $cacher->content;
      my $email = "";
      if ($content =~ /Reply to: <[^>]+?>(.+?)<\/a>/s) {
	$email = $1;
      }
      print Dumper
	({
	  Url => $url,
	  # Content => $content,
	  Email => $email,
	 }) if 0;
      $var = $email;
    } else {
      $var = $fields2->{$field}->{Default};
    }
    $fields2->{$field}->{EntryText} = \$var;
    my $fieldframe = $textframes->Frame();
    $fieldframe->Label
      (
       -text => $field,
      )->pack(-side => 'left');
    if ($fields2->{$field}->{Type} eq "large") {
      my $text = $fieldframe->Text
	(
	 -width => 80,
	 -height => 15,
	 -wrap => "word",
	);
      # render this to text
      $text->Contents(${$fields2->{$field}->{EntryText}});
      $text->pack(-side => 'right');
    } else {
      $fieldframe->Entry
	(
	 -width => length(${$fields2->{$field}->{EntryText}}),
	 -textvariable => $fields2->{$field}->{EntryText},
	)->pack(-side => 'right');
    }
    $fieldframe->pack
      (-side => "top");
  }

  my $buttonframe2 = $textframes->Frame();
  $buttonframe2->Button
    (
     -text => "Submit Application",
     -command => sub {
       SubmitApplication
	 (
	  TopLevel => $top2,
	  Order1 => $order1,
	  Fields1 => $fields1,
	  Order2 => $order2,
	  Fields2 => $fields2,
	 );
     },
    )->pack(-side => "left");
  $buttonframe2->Button
    (
     -text => "Cancel",
     -command => sub {
       # verify cancellation and then close out of top2
       # check whether the text fields have been changed from their defaults
       if (Approve("Cancel Application?")) {
	 $top2->DESTROY;
       }
     },
    )->pack(-side => "right");
  $buttonframe2->pack
    (-side => "top");
  $textframes->pack(-side => "right");
}

sub SubmitApplication {
  my (%args) = @_;
  # okay we need to generate the text and PDF versions of the cover
  # letter

  # get the text of the cover letter
  my $coverlettertext = ${$args{Fields2}->{'Cover'}->{EntryText}};
  chomp $coverlettertext;

  my $dir = tempdir("/var/lib/myfrdcsa/codebases/minor/js-rapid-response/data/responses/response-XXXXXX");

  my $coverlettertextfile = "$dir/cover-letter.txt";
  my $coverletterpdffile = "$dir/cover-letter.pdf";
  my $fh = IO::File->new;
  $fh->open(">$coverlettertextfile");
  print $fh wrap("", "", $coverlettertext);
  $fh->close();
  # generate the coverletter file
  system "/var/lib/myfrdcsa/codebases/minor/js-rapid-response/scripts/text2pdf.pl --in=$coverlettertextfile --dir=$dir/";

  # locate the correct resumes, copy them to the appropriate directory
  # generate the email text

  my $emailfile = "$dir/email.txt";
  my $emailtext = ${$args{Fields2}->{'Body:'}->{EntryText}};
  my $fh2 = IO::File->new;
  $fh2->open(">$emailfile");
  print $fh2 wrap("", "", $emailtext);
  $fh2->close();

  my @files =
    (
     $coverlettertextfile,
     $coverletterpdffile,
     "/home/andrewdo/resume/andrewdo-resume-2010-05-24T1017.txt",
     "/home/andrewdo/resume/andrewdo-resume-2010-05-24T1017.rtf",
     "/home/andrewdo/resume/andrewdo-resume-2010-05-24T1017.pdf",
    );

  # finally send the email
  my $subject = ${$args{Fields2}->{'Subject:'}->{EntryText}};
  # my $to = ${$args{Fields2}->{'To:'}->{EntryText}};
  # my $to = 'andrewdo@frdcsa.org';
  my $to = 'adougher9@yahoo.com';
  # mutt -s "Subject" -a reports.tar.gz vivek@gmail.co.in < /tmp/mail-message.txt
  my $command = "mutt -s ".shell_quote($subject)." -a ".join(" ",map {shell_quote($_)} @files)." -- ".shell_quote($to)." < $emailfile";
  # print "<$command>\n";
  my $senttimepre = time;
  ApproveCommands
    (
     Commands => [$command],
     Method => "parallel",
    );
  my $senttimepost = time;

  # record in the database that we send it
  # record the time elapsed between the posting and the sending

  # 2010-05-28T11:42:17-05:00

  my $postedtime = $args{Fields1}->{'dc:date'}->{EntryText};

  # get the date time converted, find out the time interval

  # don't play around with FreeKBS for now, just use an SQL database

  $mysql->Insert
    (
     Table => "Correspondence",
     Values => [
		From => "",
		To => "",
		URL => "",
		CoverLetter => "",
		SentTime => "",
		PostedTime => "",
	       ],
    ) if 0;

  # expect a reply

  # have a GUI for monitoring sent emails to see if we get appropriate
  # replies, also, send more letters if we haven't received a
  # response.
  $args{TopLevel}->DESTROY;
}
